/* lc_filter3.cpp   L-C filter design program  [Linux version]

****  Compile with this command (no makefile required) :

$g++ -o lc_filter3 lc_filter3.cpp

****

Revisions:
Dec 6 2012:  Added extra digit of precision to response plots.

June 14 2004: Fixed bug in highpass code that produced
nulls and negative values for poles 6,7,8,9.

May 18 2008: Fixed bug in highpass code that failed to do 1/N to table values and screwed up the values for the
"DC short to ground" version of the high pass filter.  Problem reported by Nicolae Fieraru  ( nfieraru at gmail.com )

May 12 2010: Version 2.0 - Added code to design bandpass and bandstop filters.  Cleaned up code and removed unused junk.

May 16 2010: Version 3.0 - Added code to plot frequency response.
Dec 18 2010:  Fixed but that screwed up low pass freq plots
when running on Bluehost.com servers.  (needed to init. fstart = 0; ) 


This version is designed as a CGI program to allow
it to run from a HTTP server where it can be accessed by
Web Browsers. It outputs HTML formated text. Input
is from the Browsers FORM.

Note: The input to this program comes from environment variable "QUERRY_STRING"
which is supplied by the www server.
 
 

 Copyright (C) 2009  Dale A. Heatherington
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 

*/


#include <iostream>
#include <iomanip>
#include <stdlib.h>
#include <assert.h>

#include <string>
#include <cstring>
#include <stdio.h>
#include <math.h>

using namespace std ;

#define LOWPASS 10
#define HIGHPASS 11
#define BANDPASS 12
#define BANDSTOP 13
#define HZ 1
#define KHZ 1000
#define MHZ 1000000
#define Butterworth 0
#define Chebyshev 1
#define Bessel 2


 

//--------------
// These globals supply the program input data

int filterType, poles, filterResp;


double cutoff, rs,fsf, funits, cfunits;
double fcutoff, CF, geometricCF, fcLow, fcHigh, octaves;
char ch;
char *sunits, *scfunits;


const double pi=3.141592;

const char* pF = "pF";
const char* uH = "uH";
const char* BGCOLOR="linen";
const int   nTypes = 3;
const double ripple = 0.1;
string sFilterType;
string sFreqUnits;

//Normallized filter design tables. 
//Data from Electronic Filter Design Handbook (ISBN 0-07-070430-9)

double butter3[]     = {1.0,2.0,1.0};                 //page 12-2
double cheby01_3[]   = {1.4328, 1.5937, 1.4328};   //page 12-27
double bessel3[]     = {0.3374, 0.9705, 2.2034};    //page 12-46

double butter5[]     = {0.618,1.618,2.0,1.618,0.618};           //page 12-3
double cheby01_5[]   = {1.3013, 1.5559, 2.2411,1.5559, 1.3013}; //Page 12-28
double bessel5[]     = {0.1743, 0.5072, 0.804, 1.111, 2.2582};  //page 12-47

double butter7[]     = {0.445, 1.247, 1.8019, 2.0, 1.8019, 1.247, 0.445}; //page 12-3
double cheby01_7[]   = {1.2615, 1.5195, 2.2392, 1.6804, 2.2392, 1.5196, 1.2615}; // 12-28
double bessel7[]     = {0.1106, 0.3259, 0.5249, 0.702, 0.869, 1.1052, 2.2659}; //12-47


double butter9[]     = {0.3473, 1.0, 1.5321, 1.8794, 2.0, 1.8794, 1.5321, 1.0, 0.3473}; //page 12-4
double cheby01_9[]   = {1.2446, 1.5017, 2.222, 1.6829, 2.2957, 1.6829, 2.222, 1.5017, 1.2446};// 12-29
double bessel9[]     = {0.078, 0.2313, 0.377, 0.5108, 0.6306, 0.7407, 0.8639, 1.0863, 2.2649};//12-48


//----------------------------------------------------------
void printRow(int n, const char* s,double* v, const char* units)
{  int i;
   
   cout  << "<tr><td>" << s << "</td>";
   

   for(i=0;i<n;i++){
		if(v[i] < 1){ cout << setprecision(4) << showpoint ;}
			else { cout << setprecision(2) << showpoint ; }
		if(v[i] >= 1000) { cout << setprecision(0) << noshowpoint ;}
		
      cout  << "<td>" << v[i] << " " << units << "</td>";
   }

   cout  << "</tr>";
}
//-------------------------------------------------------------
void printHeader(void)
{
   cout  << "<table border bgcolor=" << BGCOLOR << ">"
         << "<tr><td></td><td colspan=" << nTypes << "><center>Part Values</td></tr>"
         << "<tr><td>Part</td><td>Butterworth</td><td>Chebyshev 0.1 DB</td>"
         << "<td>Bessel</td></tr>" 
         << setiosflags(ios::showpoint | ios::fixed)  
		 
         << setiosflags(ios::right)
         << setprecision(3)
         << flush;
}
//--------------------------------------------------------------

//Compute inductor value to resonate at frequency F with capacitor C.
double calcInd(double F, double C)
{
	
	
	double l = 1 / (4*pi*pi*F*F*C);
	return l;
}

//---------------------------------------------------------------

//Compute capacitor value to resonate at frequency F with inductor L
double calcCap(double F, double L)
{
	
	double c = 1 / (4*pi*pi*F*F*L);
	return c;
}
//--------------------------------------------------------------
double asinh(double x)
{
	return log( x + sqrt(1 + x*x) );
	
}
//-------------------------------------------------------------
void PlotResponse(int Type, int Resp, int order, double cutoff, double cf, double funits, double cfunits)
{  double stepsize;
   int steps;
	double fstart, f, O, E, A , U, df, dfLow, fudge;
	double DBloss, bw, Omega;
	double chebypoly;
	int i;
	bool baseband, Inf;
	double plotwidth;
	
	double s = pow(10,0.1*0.1) ;  // 0.1db ripple for Chebyshev
	E = sqrt( s - 1.0) ;  // equ 2-37 in Filter Handbook for Chebyshev filters
	A = (1/(double)order) * asinh(1/E);			// equ 2-36 
	
	fstart = 0;
	steps = 60;
	plotwidth = cutoff * 4  ;
    if((Type == BANDSTOP) || (Type == HIGHPASS))  plotwidth = cutoff * 2;   //For better display of Bandstop filters
   
	stepsize = plotwidth / steps ;
	baseband = true;  //Low or high pass
	bw = cutoff;
    
    octaves = log(fcHigh/fcLow) / log(2);        //Compute bandwidth in octaves
    fudge = 1 - (octaves * 0.15);               //fudge factor to make plots of very wide bandpass filters more accurate
	
	cout << "<hr><pre>" << setprecision(4) << endl ;    //Preformatted mode for web page
    
    	/* Debug */
	/*
	cout << setprecision(4)  << "Type=" << Type << "  Resp=" << Resp 
	<< " Order=" << order << " Cutoff=" << cutoff 
	<< " Center F="  << cf << "  funits=" << funits
	<<  "  cfunits=" << cfunits 
	<<  "  E=" << E << "   A=" << A  << "  asinh(1/E)=" << asinh(1/E) << endl << endl;
	 */
	
    
	switch(Resp){
		case Butterworth:  cout << "Butterworth " ; break;
		case Chebyshev:	cout << "Chebyshev "  ; break;
		case Bessel:		cout << "Bessel "	; break;
	}
		

	
	if(Type == BANDPASS) {
        
        stepsize = plotwidth / steps ;
        if(stepsize >= 10) stepsize = round(stepsize);
        f = cf - (plotwidth / 2);
        if(f < 0) f = 0;
 
        
		baseband = false;  //Bandpass or bandstop
        

		cout << order*2 << " pole " << sFilterType << " filter frequency vs loss plot" << endl;
		U = cfunits;
	}
       
       if(Type == BANDSTOP){
           f = cf - (plotwidth / 2);
           if(f < 0) f = 0;
           baseband = false;  //Bandpass or bandstop
           cout << order*2 << " pole " << sFilterType << " filter frequency vs loss plot" << endl;
           U = cfunits;
       }
    
    if((Type == LOWPASS) || (Type == HIGHPASS)){
		f = fstart;         //Low or High pass
		cout << order << " pole " << sFilterType << " filter frequency vs loss plot" << endl;
		U = funits;
		baseband = true;
	}
	
	if(Resp == Bessel) cout << "Note: Bessel approximation may not be accurate beyond 2x cutoff frequency" << endl;	

	
	if(U == MHZ) sFreqUnits = "MHz";
	if(U == KHZ) sFreqUnits = "kHz";
	if(U == HZ)  sFreqUnits = " Hz";
	
	
	//Label plot columns
	cout  << setw(72) << sFreqUnits
	<< setw(9) << "DB loss" << endl;
	
	Inf = false;
	// Compute loss for each step frequency
	for(i=0;i<steps;i++){
		
		Inf = false;  //Assume not infinity
		
		switch(Type) {
			case LOWPASS:    Omega = f / cutoff ; 
				break;
			case HIGHPASS:	if(f > 0) Omega = cutoff / f; else Inf = true;
				break;
				
            case BANDPASS:	if(f > 0){
                                df = f-cf;
                                dfLow = ((geometricCF*geometricCF)/f) - geometricCF;
                
                                if(f >= cf) Omega =  2 * fudge *  (df/ bw)  ;
                                else  Omega = 2 * fudge * dfLow/bw ;
                
                            }else Inf = true;
                
							
				break;
				
			case BANDSTOP:	df = f - cf;
                            dfLow = ((cf*cf)/f) - cf; //Not using geometric center because user specified center freq is more important for notch.
							
                            if(df != 0) {
                                if(f > cf) Omega = 0.5 * (1/fudge) * (bw / df);
                                else  Omega = 0.5 * (1/fudge) * (bw / dfLow);
                                
                                    
                            }else Inf = true;
							
				break;
		}
				
				
				
				
		if(Resp == Butterworth){   //Compute Butterworth loss
			if(Inf == false)
				DBloss = 10 * log10((double)1 + pow(Omega,2*order)) ;  //Equ 2-32 in Filter Handbook
			else
				DBloss = INFINITY;
			
		}
		
		
		
		if(Resp == Chebyshev){    //Compute Chebyshev loss  ( This is way too complicated!!   )
			
			if(Inf == false) {
				Omega = cosh(A) * Omega;   
			
				//Table 2-1 in Filter Handbook "Chebyshev Polynomials".
				switch(order){   
				case 3:		chebypoly = (4*pow(Omega,3)) - (3*Omega);
							break;
				case 5:		chebypoly = (16*pow(Omega,5)) - (20*pow(Omega,3)) + (5*Omega);
							break;
				case 6:		chebypoly = (32*pow(Omega,6)) - (48*pow(Omega,4)) + (18*pow(Omega,2)) - 1;
							break;
																				
 										
				case 7:		chebypoly = (64*pow(Omega,7)) - (112*pow(Omega,5)) + (56*pow(Omega,3)) - (7*Omega);
							break;
															
				case 9: 	chebypoly = (256*pow(Omega,9)) - (576*pow(Omega,7)) + (432*pow(Omega,5)) - (120*pow(Omega,3)) + (9*Omega);
							break;
				case 10:	chebypoly = (512*pow(Omega,10)) - (1280*pow(Omega,8)) + (1120*pow(Omega,6)) - (400*pow(Omega,4)) + (50*pow(Omega,2)) - 1;
							break;
															
				}
										
				DBloss = 10 * log10(1 + (E*E*chebypoly*chebypoly));  //equ 2-38 
			}else
				DBloss = INFINITY;
			
			
			
		}
		
		if(Resp == Bessel){    // Compute Bessel loss
			
			if(Inf == false) 
				DBloss = 3 * pow(Omega, 2);    //This is the same for all values of filter order
			else
				DBloss = INFINITY;
			
			
		}
		
		int LOSS;
		if(DBloss > 65) LOSS = 65; else LOSS = (int)DBloss;
		   
		// Print loss vs frequency graph.
		int p = 2;

		if(!baseband && (cf/cfunits < 10)) p = 3;   //Add extra digit of precision to numbers less than 10
		double scale;
                if(!baseband) scale = cfunits; else scale = funits;
                if(baseband && (cutoff < 10)) p = 3;
		
        
		cout << setw(LOSS+1) << "*"
		<< setprecision(p)
		<< setw(72-LOSS) << f/scale
		<< setw(7) << setprecision(1) << DBloss << endl;
		
        
		f = f + stepsize;
		
	}
							
		
	cout << "</pre>" ;
	
	
}
//-------------------------------------------------------------


void ComputeHighpass(int poles)
{
 
  const char *sch1, *sch2;
	
  double* N[nTypes];
  double l1[nTypes],l2[nTypes],l3[nTypes],l4[nTypes],l5[nTypes] ;
  double c1[nTypes],c2[nTypes],c3[nTypes],c4[nTypes],c5[nTypes] ;


  const double Lscale = 1e6;
  const double Cscale = 1e12;

  
 N[0] = NULL;
 N[1] = NULL;
 N[2] = NULL;
  
 switch(poles){
 case   3: sch1 = "lchpf3.gif";
           sch2 = "lchpf3b.gif";
           N[0] = butter3;
           N[1] = cheby01_3;
           N[2] = bessel3;

           break;

 case   5: sch1 = "lchpf5.gif";
           sch2 = "lchpf5b.gif";
           N[0] = butter5;
           N[1] = cheby01_5;
           N[2] = bessel5;
           break;

 case   7: sch1 = "lchpf7.gif";
           sch2 = "lchpf7b.gif";
           N[0] = butter7;
           N[1] = cheby01_7;
           N[2] = bessel7;
           break;

 case   9: sch1 = "lchpf9.gif";
           sch2 = "lchpf9b.gif";
           N[0] = butter9;
           N[1] = cheby01_9;
           N[2] = bessel9;
           break;
 }

  fsf = 2*pi*cutoff;
  
  
  cout   << "<H3>" << poles << " pole highpass filter</h3>"
    	   << setiosflags(ios::showpoint | ios::fixed) << setprecision(1) 
		   << setiosflags(ios::left)
		   << "I/O impedcance   = " << rs << " ohms<br>"
		   << "3 DB cutoff freq = " << fcutoff << " " << sunits << "<br><hr>";
         
         
         
  cout   << "<h4>This configuration presents a DC short to ground at it's input and output</h4>"
		   << "<img src=\"/filter/" << sch2 << "\"> <p>";
              
  
   

   for(int i=0;i<nTypes;i++){

      l1[i] = Lscale * ((1/N[i][0])*rs) / fsf;
      l2[i] = Lscale * ((1/N[i][2])*rs) / fsf;
      if(poles > 3) l3[i] = Lscale * ((1/N[i][4])*rs) / fsf;
      if(poles > 5) l4[i] = Lscale * ((1/N[i][6])*rs) / fsf;
      if(poles > 7) l5[i] = Lscale * ((1/N[i][8])*rs) / fsf;

      c1[i] = Cscale * (1/N[i][1]) / (fsf*rs);
      if(poles > 3) c2[i] = Cscale * (1/N[i][3]) / (fsf*rs);
      if(poles > 5) c3[i] = Cscale * (1/N[i][5]) / (fsf*rs);
      if(poles > 7) c4[i] = Cscale * (1/N[i][7]) / (fsf*rs);
      
   }

   printHeader();
   printRow(nTypes,"L1",l1,uH);
   printRow(nTypes,"L2",l2,uH);
   if(poles > 3) printRow(nTypes,"L3",l3,uH);
   if(poles > 5) printRow(nTypes,"L4",l4,uH);
   if(poles > 7) printRow(nTypes,"L5",l5,uH);

   
   
   printRow(nTypes,"C1",c1,pF);
   if(poles > 3) printRow(nTypes,"C2",c2,pF);
   if(poles > 5) printRow(nTypes,"C3",c3,pF);
   if(poles > 7) printRow(nTypes,"C4",c4,pF);
   
   cout   << "</center></table>\n";
 
   


   cout  << "<hr><h4>This configuration presents a DC open circuit to ground for input and output</h4><br>"
        << "<img src=\"/filter/" << sch1 << "\"> <p>";

   for(int i=0;i<nTypes;i++){
  
      l1[i] = Lscale * ((1/N[i][1]) * rs) / fsf;
      if(poles > 3) l2[i] = Lscale * ((1/N[i][3]) * rs) / fsf;
      if(poles > 5) l3[i] = Lscale * ((1/N[i][5]) * rs) / fsf;
      if(poles > 7) l4[i] = Lscale * ((1/N[i][7]) * rs) / fsf;
      
      c1[i] = Cscale * (1/N[i][0]) / (fsf*rs);
      c2[i] = Cscale * (1/N[i][2]) / (fsf*rs);
      if(poles > 3) c3[i] = Cscale * (1/N[i][4]) / (fsf*rs);
      if(poles > 5) c4[i] = Cscale * (1/N[i][6]) / (fsf*rs);
      if(poles > 7) c5[i] = Cscale * (1/N[i][8]) / (fsf*rs);
      
   }

  
  printHeader();
  printRow(nTypes,"L1",l1,uH);
  if(poles > 3) printRow(nTypes,"L2",l2,uH);
  if(poles > 5) printRow(nTypes,"L3",l3,uH);
  if(poles > 7) printRow(nTypes,"L4",l4,uH);
  
  printRow(nTypes,"C1",c1,pF);
  printRow(nTypes,"C2",c2,pF);
  if(poles > 3) printRow(nTypes,"C3",c3,pF);
  if(poles > 5) printRow(nTypes,"C4",c4,pF);
  if(poles > 7) printRow(nTypes,"C5",c5,pF);

  cout   << "</center></table>\n";
    
    cout << "<pre> \nNote:  Plots below are mathematical approximations, not circuit simulations and may not be 100% accurate \n\n </pre>" ;
	
	PlotResponse(HIGHPASS,Butterworth,poles,cutoff,0,funits,funits);	
	PlotResponse(HIGHPASS,Chebyshev,poles,cutoff,0,funits,funits);
	PlotResponse(HIGHPASS,Bessel,poles,cutoff,0,funits,funits);
         
}

//-------------------------------------------------------------------------

void ComputeLowpass(int poles)
{
  const char *sch1, *sch2;
  double* N[nTypes];
  double l1[nTypes],l2[nTypes],l3[nTypes],l4[nTypes],l5[nTypes] ;
  double c1[nTypes],c2[nTypes],c3[nTypes],c4[nTypes],c5[nTypes] ;

  N[0] = NULL;
  N[1] = NULL;
  N[2] = NULL;
   
  switch(poles){
  case   3: sch1 = "lclpf3.gif";
            sch2 = "lclpf3b.gif";
            N[0] = butter3;
            N[1] = cheby01_3;
            N[2] = bessel3;

            break;

  case   5: sch1 = "lclpf5.gif";
            sch2 = "lclpf5b.gif";
            N[0] = butter5;
            N[1] = cheby01_5;
            N[2] = bessel5;
            break;

  case   7: sch1 = "lclpf7.gif";
            sch2 = "lclpf7b.gif";
            N[0] = butter7;
            N[1] = cheby01_7;
            N[2] = bessel7;
            break;

  case   9: sch1 = "lclpf9.gif";
            sch2 = "lclpf9b.gif";
            N[0] = butter9;
            N[1] = cheby01_9;
            N[2] = bessel9;
            break;
  }
  
  cout   << "<H3>" << poles << " pole lowpass filter</h3>"
    	   << setiosflags(ios::showpoint | ios::fixed) << setprecision(1) 
		   << setiosflags(ios::left)
		   << "I/O impedcance   = " << rs << " ohms<br>"
		   << "3 DB cutoff freq = " << fcutoff << " " << sunits << "<br><hr>"
         << "<h4>Best for medium to high impedance loads.</h4>"
		   << "<img src=\"/filter/" << sch1 << "\"> <p>";
              
  const double Lscale = 1e6;
  const double Cscale = 1e12;

   fsf = 2*pi*cutoff;

   for(int i=0;i<nTypes;i++){

      l1[i] = Lscale * (N[i][1]*rs) / fsf;
      if(poles > 3) l2[i] = Lscale * (N[i][3]*rs) / fsf;
      if(poles > 5) l3[i] = Lscale * (N[i][5]*rs) / fsf;
      if(poles > 7) l4[i] = Lscale * (N[i][7]*rs) / fsf;
      c1[i] = Cscale * N[i][0] / (fsf*rs);
      c2[i] = Cscale * N[i][2] / (fsf*rs);
      if(poles > 3) c3[i] = Cscale * N[i][4] / (fsf*rs);
      if(poles > 5) c4[i] = Cscale * N[i][6] / (fsf*rs);
      if(poles > 7) c5[i] = Cscale * N[i][8] / (fsf*rs);
   }

   
   printHeader();
   printRow(nTypes,"L1",l1,uH);
   if(poles > 3) printRow(nTypes,"L2",l2,uH);
   if(poles > 5) printRow(nTypes,"L3",l3,uH);
   if(poles > 7) printRow(nTypes,"L4",l4,uH);

   
   
   printRow(nTypes,"C1",c1,pF);
   printRow(nTypes,"C2",c2,pF);
   if(poles > 3) printRow(nTypes,"C3",c3,pF);
   if(poles > 5) printRow(nTypes,"C4",c4,pF);
   if(poles > 7) printRow(nTypes,"C5",c5,pF);
 
   cout   << "</center></table>\n";


 
  
   cout  << "<hr><h4>Best for low impedance loads.</h4><br>"
         << "<img src=\"/filter/" << sch2 << "\"> <p>";

   for(int i=0;i<nTypes;i++){
  
      l1[i] = Lscale * (N[i][0] * rs) / fsf;
      l2[i] = Lscale * (N[i][2] * rs) / fsf;
      if(poles > 3) l3[i] = Lscale * (N[i][4] * rs) / fsf;
      if(poles > 5) l4[i] = Lscale * (N[i][6] * rs) / fsf;
      if(poles > 7) l5[i] = Lscale * (N[i][8] * rs) / fsf;
      c1[i] = Cscale * N[i][1] / (fsf*rs);
      if(poles > 3) c2[i] = Cscale * N[i][3] / (fsf*rs);
      if(poles > 5) c3[i] = Cscale * N[i][5] / (fsf*rs);
      if(poles > 7) c4[i] = Cscale * N[i][7] / (fsf*rs);
   }


   
  printHeader();
  
  printRow(nTypes,"L1",l1,uH);
  printRow(nTypes,"L2",l2,uH);
  if(poles > 3) printRow(nTypes,"L3",l3,uH);
  if(poles > 5) printRow(nTypes,"L4",l4,uH);
  if(poles > 7) printRow(nTypes,"L5",l5,uH);
  
  printRow(nTypes,"C1",c1,pF);
  if(poles > 3) printRow(nTypes,"C2",c2,pF);
  if(poles > 5) printRow(nTypes,"C3",c3,pF);
  if(poles > 7) printRow(nTypes,"C4",c4,pF);
  cout   << "</center></table>\n";
    
    cout << "<pre> \nNote:  Plots below are mathematical approximations, not circuit simulations and may not be 100% accurate \n\n </pre>" ;
	
	PlotResponse(LOWPASS,Butterworth,poles,cutoff,0,funits,funits);
	
	PlotResponse(LOWPASS,Chebyshev,poles,cutoff,0,funits,funits);
	PlotResponse(LOWPASS,Bessel,poles,cutoff,0,funits,funits);
	
}

//----------------------------------------------------------
//Bandpass is done by  first designing a low pass filter with a cutoff frequency
//equal to the desired bandpass bandwidth then resonating all the caps and inductors with
//additional inductors and caps to the desired center frequency.
void ComputeBandpass(int poles)
{
   	
	const char *sch1, *sch2;
	double* N[nTypes];
	double l1[nTypes],l2[nTypes],l3[nTypes],l4[nTypes],l5[nTypes] ;
	double c1[nTypes],c2[nTypes],c3[nTypes],c4[nTypes],c5[nTypes] ;
    int err = 0;
    
    
    double cf = CF * cfunits;  //Center frequency in HZ
    
    fcLow = cf - cutoff/2;
    if(fcLow < 0) err = 1;              //Error if negative frequency
    fcHigh = cf + cutoff/2;
    geometricCF = sqrt(fcLow*fcHigh);
    
    cf = geometricCF;
    
    if(err == 0){
        if((fcHigh/fcLow) > 16) err = 1;    // Error if too wide > 4 octaves
    }
    
    
	
	N[0] = NULL;
	N[1] = NULL;
	N[2] = NULL;
	
	
	switch(poles){
		case   6: 			
			sch1 = "lcbpf6.gif";
            sch2 = "lcbpf6b.gif";
			N[0] = butter3;
			N[1] = cheby01_3;			
            N[2] = bessel3;			
            break;
			
			
			
		case   10: 
			
			sch1 = "lcbpf10.gif";
            sch2 = "lcbpf10b.gif";
            N[0] = butter5;
            N[1] = cheby01_5;
            N[2] = bessel5;
            break;
	}
			
	
    const double Lscale = 1e6;
    const double Cscale = 1e12;
    
   
    
    fsf = 2*pi*cutoff;
			
	cout   << "<H3>" << poles << " pole bandpass filter</h3>"
	<< setiosflags(ios::showpoint | ios::fixed) << setprecision(1) 
	<< setiosflags(ios::left)
	<< "I/O impedcance   = " << rs << " ohms<br>"
	<< "Center frequency = " << CF << " " << scfunits << "<br>"
	<< "3 DB bandwidth = " << fcutoff << " " << sunits << "<br>"
    << "Geometric center freq = " << geometricCF/cfunits << " " << scfunits << "<br>3dB Cutoff low = " << fcLow/cfunits << " " << scfunits << "<br>3dB Cutoff high = " << fcHigh/cfunits << " " << scfunits << "<hr><br>" ;
    
	
	
    if(err > 0){
        cout << "<p>Error: Bandpass filter is too wide.  Please cascade a lowpass and highpass filter.  </p>" ;
        return;
    }
    
    cout << "<h4>Circuit 1.  Low Z on I/O for signals outside passband </h4>"
        << "<img src=\"/filter/" << sch1 << "\"> <p>";
	
	for(int i=0;i<nTypes;i++){  //Compute part values
		
		//First do the lowpass prototype values
		l2[i] = Lscale * (N[i][1]*rs) / fsf;		
		c1[i] = Cscale * N[i][0] / (fsf*rs);
		c3[i] = Cscale * N[i][2] / (fsf*rs);
		if(poles == 10) {
			l4[i] = Lscale * (N[i][3]*rs) / fsf;
			c5[i] = Cscale * N[i][4] / (fsf*rs);
		}
		
		//Now resonate each part at the center frequency to create a bandpass function.
		
		l1[i] = Lscale * calcInd(cf, (c1[i]/Cscale));
		l3[i] =  Lscale * calcInd(cf, (c3[i]/Cscale));
		c2[i] =  Cscale * calcCap(cf, (l2[i] / Lscale));
		
		if(poles == 10){
			c4[i] = Cscale * calcCap(cf, (l4[i]/Lscale));
			l5[i] =  Lscale * calcInd(cf, (c5[i]/Cscale));
		}
		
		
	}
	
	
	printHeader();
	
	printRow(nTypes,"L1",l1,uH);
	printRow(nTypes,"C1",c1,pF);
	printRow(nTypes,"L2",l2,uH);
	printRow(nTypes,"C2",c2,pF);
	printRow(nTypes,"L3",l3,uH);
	printRow(nTypes,"C3",c3,pF);



	if(poles == 10) {
		
		printRow(nTypes,"L4",l4,uH);
		printRow(nTypes,"C4",c4,pF);
		printRow(nTypes,"L5",l5,uH);
		printRow(nTypes,"C5",c5,pF);
	}

		
	
	
	
		
	cout   << "</center></table>\n";
	
	
	
	
	cout  << "<hr><h4>Circuit 2.  High Z on I/O for signals outside passband.</h4><br>"
	<< "<img src=\"/filter/" << sch2 << "\"> <p>";
	
	for(int i=0;i<nTypes;i++){
		
		l1[i] = Lscale * (N[i][0] * rs) / fsf;
		l3[i] = Lscale * (N[i][2] * rs) / fsf;
		c2[i] = Cscale * N[i][1] / (fsf*rs);
		if(poles == 10){
			c4[i] = Cscale * N[i][3] / (fsf*rs);
			l5[i] = Lscale * (N[i][4] * rs) / fsf;
		}
			
			
			//Now resonate each part at the center frequency to create a bandpass function.
			
			l2[i] = Lscale * calcInd(cf, (c2[i]/Cscale));
			c1[i] =  Cscale * calcCap(cf, (l1[i] / Lscale));
			c3[i] =  Cscale * calcCap(cf, (l3[i] / Lscale));
			if(poles == 10){
				c5[i] = Cscale * calcCap(cf, (l5[i]/Lscale));
				l4[i] =  Lscale * calcInd(cf, (c4[i]/Cscale));
			}
			
		
	}
	
	
	
	printHeader();
	
	printRow(nTypes,"L1",l1,uH);
	printRow(nTypes,"C1",c1,pF);
	printRow(nTypes,"L2",l2,uH);
	printRow(nTypes,"C2",c2,pF);
	printRow(nTypes,"L3",l3,uH);
	printRow(nTypes,"C3",c3,pF);	
	if(poles == 10) {		
		printRow(nTypes,"L4",l4,uH);
		printRow(nTypes,"C4",c4,pF);
		printRow(nTypes,"L5",l5,uH);
		printRow(nTypes,"C5",c5,pF);
	}
	
	
	
	cout   << "</center></table>\n";
	
	cout << "<pre> Note:  Plots below are mathematical approximations, not circuit simulations and may not be 100% accurate \n\n </pre>" ;
    
	PlotResponse(BANDPASS,Butterworth,poles/2,cutoff,geometricCF,funits,cfunits);
	PlotResponse(BANDPASS,Chebyshev,poles/2,cutoff,geometricCF,funits,cfunits);
	PlotResponse(BANDPASS,Bessel,poles/2,cutoff,geometricCF,funits,cfunits);
}




//------------------------------------------------------------

//Band Stop is done by  first designing a high pass filter with a cutoff frequency
//equal to the desired bandstop-width then resonating all the caps and inductors with
//additional inductors and caps to the desired center frequency.

void ComputeBandstop(int poles)

{
   	
	const char *sch1, *sch2;
	double* N[nTypes];
	double l1[nTypes],l2[nTypes],l3[nTypes],l4[nTypes],l5[nTypes] ;
	double c1[nTypes],c2[nTypes],c3[nTypes],c4[nTypes],c5[nTypes] ;
    int err = 0;
	
	double cf = CF * cfunits;  //Center frequency in HZ
	
	N[0] = NULL;
	N[1] = NULL;
	N[2] = NULL;
    
    
    
    fcLow = cf - cutoff/2;
    if(fcLow < 0) err = 1;              //Error if negative frequency
    fcHigh = cf + cutoff/2;
    geometricCF = sqrt(fcLow*fcHigh);
    
//Note: We will not use geometric center frequency for calculation because we want max attenuation at the user specified center frequency.
    
    if(err == 0){
        if((fcHigh/fcLow) > 16) err = 1;    // Error if too wide > 4 octaves
    }

		
	switch(poles){
		case   6: 			
			sch1 = "lcbsf6.gif";
            sch2 = "lcbsf6b.gif";
			N[0] = butter3;
			N[1] = cheby01_3;			
            N[2] = bessel3;			
            break;
			
			
			
		case   10: 
			
			sch1 = "lcbsf10.gif";
            sch2 = "lcbsf10b.gif";
            N[0] = butter5;
            N[1] = cheby01_5;
            N[2] = bessel5;
            break;
	}
	
	
	
	cout   << "<H3>" << poles << " pole bandstop filter</h3>"
	<< setiosflags(ios::showpoint | ios::fixed) << setprecision(1) 
	<< setiosflags(ios::left)
	<< "I/O impedcance   = " << rs << " ohms<br>"
	<< "Center frequency = " << CF << " " << scfunits << "<br>"
	<< "3 DB bandwidth = " << fcutoff << " " << sunits << "<br><hr>"
	<< "<h4>Circuit 1. Low Z on I/O for signals in stopband.</h4>"
	<< "<img src=\"/filter/" << sch1 << "\"> <p>";
	
	const double Lscale = 1e6;
	const double Cscale = 1e12;
	
	fsf = 2*pi*cutoff;
	
	for(int i=0;i<nTypes;i++){  //Compute part values
		
		//First do the highpass prototype values
			
		l1[i] = Lscale * ((1/N[i][0])*rs) / fsf;
		l3[i] = Lscale * ((1/N[i][2])*rs) / fsf;
		c2[i] = Cscale * (1/N[i][1]) / (fsf*rs);
		if(poles == 10){
			l5[i] = Lscale * ((1/N[i][4])*rs) / fsf;
			c4[i] = Cscale * (1/N[i][3]) / (fsf*rs);
		}
			
		//Now resonate each part at the center frequency to create a bandstop function.
		l2[i] = Lscale * calcInd(cf, (c2[i]/Cscale));
		c1[i] =  Cscale * calcCap(cf, (l1[i] / Lscale));
		c3[i] =  Cscale * calcCap(cf, (l3[i] / Lscale));
		if(poles == 10){
			c5[i] = Cscale * calcCap(cf, (l5[i]/Lscale));
			l4[i] =  Lscale * calcInd(cf, (c4[i]/Cscale));
		}
		
		
		
	}
	
	
	printHeader();
	
	printRow(nTypes,"L1",l1,uH);
	printRow(nTypes,"C1",c1,pF);
	printRow(nTypes,"L2",l2,uH);
	printRow(nTypes,"C2",c2,pF);
	printRow(nTypes,"L3",l3,uH);
	printRow(nTypes,"C3",c3,pF);
	
	
	
	if(poles == 10) {
		
		printRow(nTypes,"L4",l4,uH);
		printRow(nTypes,"C4",c4,pF);
		printRow(nTypes,"L5",l5,uH);
		printRow(nTypes,"C5",c5,pF);
	}
		
	cout   << "</center></table>\n";
	cout  << "<hr><h4>Circuit 2.  High Z on I/O for signals in stopband.</h4><br>"
	<< "<img src=\"/filter/" << sch2 << "\"> <p>";
	
	for(int i=0;i<nTypes;i++){
		
		//compute highpass prototype values

		l2[i] = Lscale * ((1/N[i][1]) * rs) / fsf;		
		c1[i] = Cscale * (1/N[i][0]) / (fsf*rs);
		c3[i] = Cscale * (1/N[i][2]) / (fsf*rs);
		if(poles == 10){ 
			c5[i] = Cscale * (1/N[i][4]) / (fsf*rs);
			l4[i] = Lscale * ((1/N[i][3]) * rs) / fsf;
		}
				
			
			//Now resonate each part at the center frequency to create a bandstop function.
			
			l1[i] = Lscale * calcInd(cf, (c1[i]/Cscale));
			l3[i] =  Lscale * calcInd(cf, (c3[i]/Cscale));
			c2[i] =  Cscale * calcCap(cf, (l2[i] / Lscale));
			
			if(poles == 10){
				c4[i] = Cscale * calcCap(cf, (l4[i]/Lscale));
				l5[i] =  Lscale * calcInd(cf, (c5[i]/Cscale));
			}
			
			
		
	}
	
	printHeader();
	
	printRow(nTypes,"L1",l1,uH);
	printRow(nTypes,"C1",c1,pF);
	printRow(nTypes,"L2",l2,uH);
	printRow(nTypes,"C2",c2,pF);
	printRow(nTypes,"L3",l3,uH);
	printRow(nTypes,"C3",c3,pF);	
	if(poles == 10) {		
		printRow(nTypes,"L4",l4,uH);
		printRow(nTypes,"C4",c4,pF);
		printRow(nTypes,"L5",l5,uH);
		printRow(nTypes,"C5",c5,pF);
	}
		
	cout   << "</center></table>\n";
    
    cout << "<pre> \nNote:  Plots below are mathematical approximations, not circuit simulations and may not be 100% accurate \n\n </pre>" ;
	
	PlotResponse(BANDSTOP,Butterworth,poles/2,cutoff,cf,funits,cfunits);
	
	PlotResponse(BANDSTOP,Chebyshev,poles/2,cutoff,cf,funits,cfunits);
	PlotResponse(BANDSTOP,Bessel,poles/2,cutoff,cf,funits,cfunits);
	
	
}

//------------------------------------------------------------
const char *Validate(void)
{

  if (rs < 1)    return "Impedance";
  if (filterResp == 0) return "Filter Response";
  
  if ((poles > 10) || (poles < 2)) return "poles";
  if ((filterResp == BANDPASS) && (poles & 1)) return "poles odd";
  if ((filterResp == LOWPASS) && ((poles & 1) == 0)) return "poles even";
  if ((filterResp == HIGHPASS) && ((poles & 1) == 0)) return "poles even";
  if (cutoff == 0.0) return "Cutoff frequency";
  
  return NULL;

}
//------------------------------------------------------------
//query_string format :  name=value&name2=value2&name3=value3&.....name10=value10

void ProcQuery(const char *cp)
{
  char* arg;
  char* string;
  char* name;
	char query[strlen(cp)+1];  //buffer to hold query string
	
  float val=0.0;

   filterResp = 0;
   filterType = 0;
   
	strcpy(query,cp);		//Make a local copy of query string
	
	char *s = strtok(query,"&");       //s = name,value pair into s

	while(s != NULL)
    {
	  	  name = s;
		  arg = strchr(s,'=');
        *arg = (char)NULL;
        
		  arg++;
        string = arg;
        //cout << name << "  " << string << "<br>\n";
        val = 0.0;
		  sscanf(arg,"%f",&val);
		
		if (strcmp(name,"FilterResponse") == 0){
			if(strcmp(string,"Lowpass") == 0) filterResp = LOWPASS;
			if(strcmp(string,"Highpass") == 0) filterResp = HIGHPASS;
			if(strcmp(string,"Bandpass") == 0) filterResp = BANDPASS;
			if(strcmp(string,"Bandstop") == 0) filterResp = BANDSTOP;
			
        }
		
		

        if (strcmp(name,"cutoff") == 0)   fcutoff = (double)val; //3 db cutoff freq
		
        if (strcmp(name,"funits") == 0){				//Cut off frequency units
             if(strcmp(string,"HZ" ) == 0) funits = HZ;
             if(strcmp(string,"KHZ") == 0) funits = KHZ;
             if(strcmp(string,"MHZ") == 0) funits = MHZ;
             sunits = strdup(string);

        }
		
		if (strcmp(name,"cfunits") == 0){				//Center frequency units
			if(strcmp(string,"HZ" ) == 0) cfunits = HZ;
			if(strcmp(string,"KHZ") == 0) cfunits = KHZ;
			if(strcmp(string,"MHZ") == 0) cfunits = MHZ;
			scfunits = strdup(string);
			
        }
		if (strcmp(name, "CF") == 0)     CF = (double)val;  //Center frequency
		

        if (strcmp(name,"Z") == 0)        rs = (double)val;     //I/O impedance
        
        if (strcmp(name,"poles") == 0)   poles = (int)val;		//Number of poles
        
        
			s = strtok(NULL,"&");		 //get next name,value pair
    }

   cutoff = fcutoff * funits;   //Bandwidth for bandpass filter
}

//-------------------------------------------------------------


//This is a http CGI program.  Input is from the environment variable QUERY_STRING.

int main(int argc, char *argv[])
{

  const char *cp;

  const char *query_string = getenv("QUERY_STRING");
  if (query_string == NULL) query_string = "";

	cout  << "Content-Type: text/html"
        << endl 
        << endl;
	
	



        ProcQuery(query_string);       //Parse the query string
        cp = Validate();              //validate user input

        if ( cp == NULL)
           {  cout << "<html><body bgcolor=\"#ffffff\"> <p>V4.0</p>" ;
			   
              
		       //Compute filter parts values
              switch(filterResp){
              case LOWPASS:		sFilterType = "Lowpass";
								ComputeLowpass(poles);
								break; 

              case HIGHPASS:	sFilterType = "Highpass";
								ComputeHighpass(poles);
								break;

              case BANDPASS:	sFilterType = "Bandpass";
								ComputeBandpass(poles);
								break;
					  
			  case BANDSTOP:	sFilterType = "Bandstop";
								ComputeBandstop(poles);
								break;
					  
				 

              default:        cout << "Invalid Filter Response Function Requested<br>\n";
                              break;
              }


              cout  << "<hr></body></html>";
		
           }
           else
              cout  << "<br><H3>Invalid data entered: " << cp << " error</H3>\n";
     


  
  return 0;
}






